# $Id: multileveldrilldown.py 3919 2010-08-31 06:51:38Z place $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.4.3.0/src/delfttilburg/multileveldrilldown.py $
## begin license ##
#
#    "Delft-Tilburg" (delfttilburg) is a package containing shared code
#    between the Delft "Discover" and Tilburg "Beter Zoeken & Vinden" projects.
#    Both projects are based on Meresco Software (http://meresco.com)
#    Copyright (C) 2008-2009 Technische Universiteit Delft http://www.tudelft.nl
#    Copyright (C) 2008-2009 Universiteit van Tilburg http://www.uvt.nl
#    Copyright (C) 2008-2009 Seek You Too (CQ2) http://www.cq2.nl
#
#    This file is part of "Delft-Tilburg".
#
#    "Delft-Tilburg" is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    "Delft-Tilburg" is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with "Delft-Tilburg"; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##
import pprint
import traceback

from meresco.core import Observable

class MultiLevelDrilldownException(Exception):
    pass

class MultiLevelDrilldown(Observable):
    def __init__(self, infra):
        self.infra = infra
        self.logger = infra.logging.getLogger('meresco.multileveldrilldown')
        self.pp = pprint.PrettyPrinter(indent=2)
        self.fieldFeatures = {}
        Observable.__init__(self)

    def multiLevelDrilldown(self, docset, drilldownFields, remoteMultiLevelDrilldownDict=None, remoteFieldFeatures=None, method="min", minimumTermCount=None):
        self.logger.debug("starting logger")

        if remoteFieldFeatures is None:
            fieldFeatures = self.any.makeDrilldownDict()
        else:
            fieldFeatures = remoteFieldFeatures; 
        
        if remoteMultiLevelDrilldownDict is None:    
            multiLevelDrilldownDict = self.any.getMultiLevelDrilldownFields()
        else:
            multiLevelDrilldownDict = remoteMultiLevelDrilldownDict

        #self.logger.debug( "multiLevelDrilldownDict: " + self.pp.pformat( multiLevelDrilldownDict))

        for field in drilldownFields:
            if field not in multiLevelDrilldownDict:
                raise MultiLevelDrilldownException("Unrecognized drilldownfield: file:section:'%s'" % field)

            resultFieldName, resultTermCounts = None, []
            if method == "min":
                yield self.minMethod(levelFields=multiLevelDrilldownDict[field], fieldFeatures=fieldFeatures, docset=docset, field=field, minimumTermCount=minimumTermCount)
            else: yield self.maxMethod(levelFields=multiLevelDrilldownDict[field], fieldFeatures=fieldFeatures, docset=docset, field=field)

    def minMethod(self, levelFields=[], fieldFeatures=None, docset=None, field='', minimumTermCount=None):
        self.logger.debug("minMethod, minimumTermCount=" + minimumTermCount)
        resultFieldName = ''
        resultTermCounts = 0
        for levelField in levelFields:
            maximumCount, maxDisplay, sorted = fieldFeatures[levelField]
            self.logger.debug('maximumCount=' +str(maximumCount) + ' sorted: "' + str(sorted) + '"' )
            fieldName, termCounts = self.any.drilldown(docset, [(levelField, maximumCount, sorted)]).next()
            termCounts = list(termCounts)
            numberOfTerms = len(termCounts)
            self.logger.debug('fieldName: "' + str(fieldName) + '" numberOfTerms=' + str(numberOfTerms) +'' )
                
            if numberOfTerms > 0:
                resultFieldName, resultTermCounts = fieldName, termCounts
            if numberOfTerms >= int(minimumTermCount):
                break

        self.logger.debug('field: "' + str(field) + '" resultFieldName: "' +  str(resultFieldName) + '" resultTermCounts=' + str(resultTermCounts))
        return (field, resultFieldName), resultTermCounts

    def maxMethod(self, levelFields=[], fieldFeatures=None, docset=None, field=''):
        self.logger.debug("maxMethod")
        for levelField in levelFields:
            maximumCount, maxDisplay, sorted = fieldFeatures[levelField]
            self.logger.debug('maximumCount: "' +str(maximumCount) + '" sorted: "' + str(sorted) + '"' )
            fieldName, termCounts = self.any.drilldown(docset, [(levelField, maximumCount, sorted)]).next()
            termCounts = list(termCounts)
            self.logger.debug('fieldName: "' + str(fieldName) + '" termCounts=' + str(termCounts))
                
            if len(termCounts) > 0:
                resultFieldName, resultTermCounts = fieldName, termCounts
            if len(termCounts) < maximumCount:
                break

        self.logger.debug('field: "' + str(field) + '" resultFieldName: "' +  str(resultFieldName) + '" resultTermCounts: "' + str(resultTermCounts) + '"')
        return (field, resultFieldName), resultTermCounts
